/***************************************************************************
 *   Copyright (C) 2007 by Holger Gerth                                    *
 *   holger.gerth@informatik.tu-chemnitz.de                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "advancedhrle.h"

CAdvancedHRLE::CAdvancedHRLE()
{
    this->partialHrleObject = NULL;

    this->minRange = CVector3D(-5.0,-5.0,-5.0);
    this->maxRange = CVector3D(5.0,5.0,5.0);
}


CAdvancedHRLE::~CAdvancedHRLE()
{
    if ( this->partialHrleObject != NULL) delete this->partialHrleObject;
}

// reload the master file after changes
bool CAdvancedHRLE::reloadObjectFile() {
    return this->loadObjectFile( this->workingDirectory );
}

// loads the master file of the data record into memory
bool CAdvancedHRLE::loadObjectFile( string directory ) {

    ifstream    masterFile;
    char*       tagName = (char*)malloc(5* sizeof(char));
    short int   iBuffer,iSize;
    int         i;
    string filename = directory;
    filename += "master.dat";

    // clear arrays so that there's no old loaded data
    this->runTypesZ.clear();
    this->runBreaksZ.clear();

    // open the master file
    masterFile.open( filename.c_str(), ios::binary & ios::in );

    if ( !masterFile.good() ) {
        cerr << "Error while opening file: " << filename << endl;
        return false;
    }
    else {
        this->workingDirectory = directory;
    }

    // read contents
    while ( masterFile.good() ) {
        masterFile.read( tagName, 4 );
        tagName[4] = '\0';

        // stop reading if nothing had been read
        if ( masterFile.gcount() == 0 ) break;

        // extends
        if ( string(tagName) == "MinX" ) {
            masterFile.read( (char*)&iBuffer, 2 );
            this->minX = iBuffer;
        }
        if ( string(tagName) == "MinY" ) {
            masterFile.read( (char*)&iBuffer, 2 );
            this->minY = iBuffer;
        }
        if ( string(tagName) == "MinZ" ) {
            masterFile.read( (char*)&iBuffer, 2 );
            this->minZ = iBuffer;
        }
        if ( string(tagName) == "MaxX" ) {
            masterFile.read( (char*)&iBuffer, 2 );
            this->maxX = iBuffer;
        }
        if ( string(tagName) == "MaxY" ) {
            masterFile.read( (char*)&iBuffer, 2 );
            this->maxY = iBuffer;
        }
        if ( string(tagName) == "MaxZ" ) {
            masterFile.read( (char*)&iBuffer, 2 );
            this->maxZ = iBuffer;
        }

        // runs of the z-axis
        if ( string(tagName) == "Runs" ) {
            masterFile.read( (char*)&iSize, 2 );

            for ( i = 0; i < iSize; i++ ) {
                masterFile.read( (char*)&iBuffer, 2 );
                this->runTypesZ.push_back( iBuffer );
            }
        }

        // breaks of the z-axis
        if ( string(tagName) == "Brks" ) {
            masterFile.read( (char*)&iSize, 2 );
            for ( i = 0; i < iSize; i++ ) {
                masterFile.read( (char*)&iBuffer, 2 );
                this->runBreaksZ.push_back( iBuffer );
            }
        }
    }

    masterFile.close();

    free(tagName);

    // setup min and max range for drawing
    this->minRange = CVector3D(-((this->maxX - this->minX)/2),-((this->maxY - this->minY)/2),-((this->maxZ - this->minZ)/2));
    this->maxRange = CVector3D((this->maxX - this->minX)/2,(this->maxY - this->minY)/2,(this->maxZ - this->minZ)/2);

//     this->printDataStructure();

    return true;
}

// loads one single xy-plane of the object into memory
CHierarchicalRLE* CAdvancedHRLE::getPlane( int definedIndex, bool backup ) {

    ifstream          partFile;
    CHierarchicalRLE* tmpHRLE = NULL;
    char*             tagName = (char*)malloc(5* sizeof(char));
    int               iBuffer,iSize;
    int               i;
    string            filename = this->workingDirectory;

    // setup filename
    filename += "part_";
    int partIndex(definedIndex);
    std::stringstream ss;
    if (ss << partIndex) filename += ss.str();
    else { cerr << "Error defining filename for partfile!" << endl; return false; }
    if (backup)
        filename += ".bak";
    else
        filename += ".dat";

    // close filestream if it is allready open
    if (partFile.is_open() ) {
        cerr << "Error!!!!! File allready open!!!" << endl;
        partFile.close();
    }

    // open part file
    partFile.open( filename.c_str(), ios::binary & ios::in );

    // corrupted file
    if ( !partFile.good() ) {
        cerr << "Error while opening file: " << filename << endl;
        return NULL;
    }

//     cout << "Reading file " << filename << endl;

    tmpHRLE = new CHierarchicalRLE( 2 );

    // get content of the part file
    while ( partFile.good() ) {

        partFile.read( (char*)tagName, 4 );
        tagName[4] = '\0';

        // stop reading if nothing had been read
        if ( partFile.gcount() == 0 ) break;

        // runs of the y-axis
        if ( string(tagName) == "RunY" ) {
            partFile.read( (char*)&iSize, 4 );

            for ( i = 0; i < iSize; i++ ) {
                partFile.read( (char*)&iBuffer, 4 );
                tmpHRLE->run_types.push_back( iBuffer );
            }
        }

        // breaks of the z-axis
        if ( string(tagName) == "BrkY" ) {
            partFile.read( (char*)&iSize, 4 );

            for ( i = 0; i < iSize; i++ ) {
                partFile.read( (char*)&iBuffer, 2 );
                tmpHRLE->run_breaks.push_back( iBuffer );
            }
        }

        // runs of the x-axis
        if ( string(tagName) == "RunX" ) {
            partFile.read( (char*)&iSize, 4 );

            for ( i = 0; i < iSize; i++ ) {
                partFile.read( (char*)&iBuffer, 4 );
                tmpHRLE->lower_hrle->run_types.push_back( iBuffer );
            }
        }

        // breaks of the x-axis
        if ( string(tagName) == "BrkX" ) {
            partFile.read( (char*)&iSize, 4 );

            for ( i = 0; i < iSize; i++ ) {
                partFile.read( (char*)&iBuffer, 2 );
                tmpHRLE->lower_hrle->run_breaks.push_back( iBuffer );
            }
        }

        // start indices of the different segments along the x-axis
        if ( string(tagName) == "iStX" ) {
            partFile.read( (char*)&iSize, 4 );

            for ( i = 0; i < iSize; i++ ) {
                partFile.read( (char*)&iBuffer, 4 );
                tmpHRLE->lower_hrle->start_indices.push_back( iBuffer );
            }
        }
    }

    partFile.close();

    free(tagName);

    // setup minimum an maximum extends of the plane and the start index of this 2D HRLE structure
    tmpHRLE->min = this->minY;
    tmpHRLE->max = this->maxY;
    tmpHRLE->lower_hrle->min = this->minX;
    tmpHRLE->lower_hrle->max = this->maxX;
    tmpHRLE->start_indices.push_back(0);

//     tmpHRLE->printDataStructure();

    return tmpHRLE;
}

// loads a part of the date in a given range of z-coordinates
bool CAdvancedHRLE::loadPartialObject( short int startCoord, short int endCoord, bool thinning, bool backup ) {

    CHierarchicalRLE* tmpHRLE = NULL;
    int               i,j;
    int               newDefinedIndex = 0, definedIndex;
    short int         runStart = this->minZ, runEnd = this->maxZ;
    short int         startOfOriginalRun = this->minZ;
    short int         endOfOriginalRun;
    int               run;

    // remove formerly loaded parts
    if ( this->partialHrleObject != NULL) {
        delete this->partialHrleObject;
        partialHrleObject = NULL;
    }

    // allocate memory
    this->partialHrleObject = new CHierarchicalRLE( 3 );

    // setup min and max for the partial object
    if ( startCoord > this->minZ )  this->partialHrleObject->min = startCoord;
    else this->partialHrleObject->min = this->minZ;
    if ( endCoord < this->maxZ ) this->partialHrleObject->max = endCoord;
    else this->partialHrleObject->max = this->maxZ;

    // start index
    this->partialHrleObject->start_indices.push_back( 0 );

    if ( this->runBreaksZ.size() < 1 ) {
        this->partialHrleObject->run_types.push_back( this->runTypesZ.at(0) );
    }
    else {
        endOfOriginalRun = this->runBreaksZ.at(0);
        for ( int i = 0; i < this->runTypesZ.size(); i++ ) {
            if ( (endOfOriginalRun > startCoord) &&
                 ((endOfOriginalRun <= endCoord ) || (endCoord > startOfOriginalRun) )) {

                // get the run
                run = this->runTypesZ.at(i); 

                // defined run
                if ( RT_DEFINED( run ) ) {
                    this->partialHrleObject->run_types.push_back( newDefinedIndex );

                    // start coordinate of the loaded part of this defined run
                    if ( startCoord > startOfOriginalRun ) runStart = startCoord;
                    else runStart = startOfOriginalRun;

                    // end coordinate of the loaded part of this defined run
                    if ( endCoord < endOfOriginalRun ) runEnd = endCoord;
                    else runEnd = endOfOriginalRun;

                    // load the xy-planes for every coordinate of the loaded part of this defined run
                    for ( j = runStart; j < runEnd; j++ ) {
                        // calculate defined index
                        definedIndex = run + j - startOfOriginalRun;
                        // get the xy-plane
                        if (thinning && (j == startCoord)) tmpHRLE = this->getPlane( definedIndex );
                        else tmpHRLE = this->getPlane( definedIndex, backup );
                        // add to the loaded z-block
                        if ( this->partialHrleObject->lower_hrle == NULL)
                            this->partialHrleObject->lower_hrle = tmpHRLE;
                        else
                            this->partialHrleObject->lower_hrle->addToHRLE( tmpHRLE );

                    }

                    newDefinedIndex += (runEnd - runStart);
                }
                // non defined run
                else {
                    this->partialHrleObject->run_types.push_back( this->runTypesZ.at(i) );
                }
            }
            if ( (endOfOriginalRun > startCoord) && (endOfOriginalRun < endCoord) && (endOfOriginalRun < this->maxZ) ) {
                this->partialHrleObject->run_breaks.push_back( endOfOriginalRun );
            }

            // first coordinate of the next run
            startOfOriginalRun = endOfOriginalRun;
            // last coordinate+1 of the next run
            if ( (i+1) >= this->runBreaksZ.size() ) endOfOriginalRun = this->maxZ;
            else endOfOriginalRun = this->runBreaksZ.at( i+1 );
        }
    }

    return true;
}

bool CAdvancedHRLE::morphOperation( CHierarchicalRLE* structureElement, vector<short int> qv, short int numZCoords, morphType type ) {

//     long            numGridPoints = 0, accessTime = 0, totalAccessTime = 0, averageAccessTime = 0;
//     struct timeval  tp1,tp2;

//     gettimeofday( &tp1, NULL );

    // save the old object
    this->backupFiles();

    short int zBlock, i, j;
    CHierarchicalRLE      *dilatedSegmentZ = new CHierarchicalRLE(3), *tmpHRLE = NULL, *tmpPart = NULL;
    short int             numZBlocks;
    short int             startCoord, endCoord, firstLoadedCoord, lastLoadedCoord, startOfRun;
    short int             rewriteStartCoord, rewriteEndCoord;
    int                   defYIndex = 0;
    short int             overlapLeft, overlapRight;

    // setup overlapping
    if ( type == DILATION ) {
        overlapLeft = structureElement->max - qv.back() -1;
        overlapRight = qv.back() - structureElement->min;
    }
    else if ( type == EROSION ) {
        overlapLeft = qv.back() - structureElement->min;
        overlapRight = structureElement->max - qv.back() -1;
    }
    else {
        short int overlapping;
        if ( (qv.back() - structureElement->min) > (structureElement->max - qv.back() -1) )
            overlapping = qv.back() - structureElement->min;
        else
            overlapping = structureElement->max - qv.back() -1;
        overlapLeft = overlapping;
        overlapRight = overlapping;
    }

    // setup min and max
    dilatedSegmentZ->min = this->minZ;
    dilatedSegmentZ->max = this->maxZ;
    dilatedSegmentZ->lower_hrle->min = this->minY;
    dilatedSegmentZ->lower_hrle->max = this->maxY;
    dilatedSegmentZ->lower_hrle->lower_hrle->min = this->minX;
    dilatedSegmentZ->lower_hrle->lower_hrle->max = this->maxX;

    if (  ((this->maxZ - this->minZ) % numZCoords) != 0 )  
        numZBlocks = (this->maxZ - this->minZ) / numZCoords + 1;
    else  
        numZBlocks = (this->maxZ - this->minZ) / numZCoords;

    // start and end coordinates of the first block
    startCoord = this->minZ;
    if (numZBlocks == 1) endCoord = this->maxZ;
    else endCoord = startCoord + numZCoords;

    // dilate the z-Blocks seperately
    for ( zBlock = 0; zBlock < numZBlocks; zBlock++ ) {

        // setup first and last coordinate which has to be loaded
        if ( (startCoord - overlapLeft) > this->minZ ) firstLoadedCoord = startCoord - overlapLeft;
        else firstLoadedCoord = startCoord;
        if ( (endCoord + overlapRight) < this->maxZ ) lastLoadedCoord = endCoord + overlapRight;
        else lastLoadedCoord = endCoord;

        if ( loadPartialObject( firstLoadedCoord, lastLoadedCoord, false, true ) ) {

            // dilate the part
            if ( type == DILATION )
                tmpHRLE = this->partialHrleObject->getDilatedHRLE( structureElement, qv );
            // erode the part
            if ( type == EROSION )
                tmpHRLE = this->partialHrleObject->getErodedHRLE( structureElement, qv );
            // opening
            if ( type == OPENING ) {
                tmpHRLE = this->partialHrleObject->getErodedHRLE( structureElement, qv );
                tmpHRLE = tmpHRLE->getDilatedHRLE( structureElement, qv );
            }
            // closing
            if ( type == CLOSING ) {
                tmpHRLE = this->partialHrleObject->getDilatedHRLE( structureElement, qv );
                tmpHRLE = tmpHRLE->getErodedHRLE( structureElement, qv );
            }


            if ( tmpHRLE != NULL ) {
                delete this->partialHrleObject;
                this->partialHrleObject = tmpHRLE;
            }

            // rewrite the loaded parts (without overlappings)
            this->rewriteLoadedParts( startCoord, endCoord, dilatedSegmentZ, &defYIndex,
                                      firstLoadedCoord, lastLoadedCoord );
        }
        else
            cout << "Error while loading partial object from zCoord " << zBlock;

        // update start and end coordinates
        startCoord = endCoord;
        if ((zBlock+2) >= numZBlocks) endCoord = this->maxZ;
        else endCoord += numZCoords;
    }

    // rewrite the master file
    this->rewriteMasterFile( dilatedSegmentZ );

//     gettimeofday( &tp2, NULL );

//     accessTime = (tp2.tv_sec -tp1.tv_sec) * 1000000 + (tp2.tv_usec - tp1.tv_usec);

//     cout << "Total time while accessing the voxel object was " << (accessTime / 1000000.0) << " seconds.";

    return true;
}

// compute medial axis of the object
bool CAdvancedHRLE::medialAxis() {

//     long            numGridPoints = 0, accessTime = 0, totalAccessTime = 0, averageAccessTime = 0;
//     struct timeval  tp1,tp2;

//     gettimeofday( &tp1, NULL );

    short int numZCoords = 10;
    short int zBlock, i, j;
    CHierarchicalRLE      *segmentZ = NULL, *tmpHRLE = NULL, *tmpPart = NULL;
    short int             numZBlocks;
    short int             startCoord, endCoord, firstLoadedCoord, lastLoadedCoord, startOfRun;
    short int             zRun, zCoord;
    int                   defYIndex;
    bool                  bTopHold, bBottomHold;
    bool                  pointsDeleted;
    int                   type;

    // first calculate the medial surface
    do {

        // direction order: B, T, E, W, S, N
        for ( type = 1; type < 7; type++ ) {

            if (  ((this->maxZ - this->minZ) % numZCoords) != 0 )
                numZBlocks = (this->maxZ - this->minZ) / numZCoords + 1;
            else
                numZBlocks = (this->maxZ - this->minZ) / numZCoords;

            // save the old object
            this->backupFiles();

            startCoord = this->minZ;
            if (numZBlocks == 1) endCoord = this->maxZ;
            else endCoord = startCoord + numZCoords;

            // setup z-segment for the master file
            segmentZ = new CHierarchicalRLE(3);
            segmentZ->min = this->minZ;
            segmentZ->max = this->maxZ;
            segmentZ->lower_hrle->min = this->minY;
            segmentZ->lower_hrle->max = this->maxY;
            segmentZ->lower_hrle->lower_hrle->min = this->minX;
            segmentZ->lower_hrle->lower_hrle->max = this->maxX;

            defYIndex = 0;

            pointsDeleted = false;

            // thin the z-Blocks seperately
            for ( zBlock = 0; zBlock < numZBlocks; zBlock++ ) {

                // setup loaded coords and hold values for border crossings
                if ( (startCoord-1) > this->minZ ) {
                    firstLoadedCoord = startCoord - 1;
                    bBottomHold = true;
                }
                else {
                    firstLoadedCoord = startCoord;
                    bBottomHold = false;
                }
                if ( (endCoord+1) < this->maxZ ) {
                    lastLoadedCoord = endCoord + 1;
                    bTopHold = true;
                }
                else {
                    lastLoadedCoord = endCoord;
                    bTopHold = false;
                }

                // load z-block and do one thinning step on it
                if ( loadPartialObject( firstLoadedCoord, lastLoadedCoord, false, true ) ) {

                    if (pointsDeleted == false)
                        pointsDeleted = this->partialHrleObject->thinObject( 2, (borderPointType)type );
                    else
                        this->partialHrleObject->thinObject( 2, (borderPointType)type );

                    this->rewriteLoadedParts( startCoord, endCoord, segmentZ, &defYIndex,
                                              firstLoadedCoord, lastLoadedCoord);

                }
                else
                    cout << "Error while loading partial object from zCoord " << zBlock;

                // update start and end coordinates for the next block
                startCoord = endCoord;
                if ((zBlock+1) >= numZBlocks) endCoord = this->maxZ;
                else endCoord += numZCoords;

            }

            this->rewriteMasterFile( segmentZ );
        }

    } while ( pointsDeleted );


    // now the same procedure for the medial axis
    do {

        // direction order: B, T, E, W, S, N
        for ( type = 1; type < 7; type++ ) {

            segmentZ = NULL;
            tmpHRLE = NULL;
            tmpPart = NULL;

            if (  ((this->maxZ - this->minZ) % numZCoords) != 0 )
                numZBlocks = (this->maxZ - this->minZ) / numZCoords + 1;
            else
                numZBlocks = (this->maxZ - this->minZ) / numZCoords;

            // save the old object
            this->backupFiles();

            // setup start and end coordinate of the z-block
            startCoord = this->minZ;
            if (numZBlocks == 1) endCoord = this->maxZ;
            else endCoord = startCoord + numZCoords;

            // setup z-segment for the master file
            segmentZ = new CHierarchicalRLE(3);
            segmentZ->min = this->minZ;
            segmentZ->max = this->maxZ;
            segmentZ->lower_hrle->min = this->minY;
            segmentZ->lower_hrle->max = this->maxY;
            segmentZ->lower_hrle->lower_hrle->min = this->minX;
            segmentZ->lower_hrle->lower_hrle->max = this->maxX;

            defYIndex = 0;

            pointsDeleted = false;

            // thin the z-Blocks seperately
            for ( zBlock = 0; zBlock < numZBlocks; zBlock++ ) {

                // setup loaded coords and hold values for border crossings
                if ( (startCoord-1) > this->minZ ) {
                    firstLoadedCoord = startCoord - 1;
                    bBottomHold = true;
                }
                else {
                    firstLoadedCoord = startCoord;
                    bBottomHold = false;
                }
                if ( (endCoord+1) < this->maxZ ) {
                    lastLoadedCoord = endCoord + 1;
                    bTopHold = true;
                }
                else {
                    lastLoadedCoord = endCoord;
                    bTopHold = false;
                }

                // load z-block and do one thinning step on it
                if ( loadPartialObject( firstLoadedCoord, lastLoadedCoord, false, true ) ) {

                    if (pointsDeleted == false)
                        pointsDeleted = this->partialHrleObject->thinObject( 1, (borderPointType)type );
                    else
                        this->partialHrleObject->thinObject( 1, (borderPointType)type );

                    this->rewriteLoadedParts( startCoord, endCoord, segmentZ, &defYIndex,
                                              firstLoadedCoord, lastLoadedCoord);

                }
                else
                    cout << "Error while loading partial object from zCoord " << zBlock;

                // update start and end coordinates for the next block
                startCoord = endCoord;
                if ((zBlock+1) >= numZBlocks) endCoord = this->maxZ;
                else endCoord += numZCoords;

            }

            this->rewriteMasterFile( segmentZ );
        }

    } while ( pointsDeleted );

//     gettimeofday( &tp2, NULL );

//     accessTime = (tp2.tv_sec -tp1.tv_sec) * 1000000 + (tp2.tv_usec - tp1.tv_usec);

//     cout << "Total time while accessing the voxel object was " << (accessTime / 1000000.0) << " seconds.";

    return true;
}

// creates a backup of the data files
bool CAdvancedHRLE::backupFiles() {

    int         i,j;
    short int   run, startOfRun, endOfRun, definedIndex;
    string inputFilename = this->workingDirectory;
    string outputFilename = this->workingDirectory;

    inputFilename += "master.dat";
    outputFilename += "master.bak";

    this->copy_file(inputFilename, outputFilename);

    startOfRun = this->minZ;
    if ( this->runBreaksZ.size() == 0 ) endOfRun = this->maxZ;
    else endOfRun = this->runBreaksZ.at(0);

    for ( i = 0; i < this->runTypesZ.size(); i++ ) {

        run = this->runTypesZ.at(i);

        if ( RT_DEFINED( run ) ) {

            for ( j = startOfRun; j < endOfRun; j++ ) {

                definedIndex = run + j - startOfRun;

                inputFilename = this->workingDirectory;
                inputFilename += "part_";
                int partIndex(definedIndex);
                std::stringstream ss;
                if (ss << partIndex) inputFilename += ss.str();
                else { cerr << "Error defining filename for partfile!" << endl; return false; }
                outputFilename = inputFilename;
                inputFilename += ".dat";
                outputFilename += ".bak";

                this->copy_file(inputFilename, outputFilename);

            }

        }

        startOfRun = endOfRun;
        if ( (i+1) >= this->runBreaksZ.size() ) endOfRun = this->maxZ;
        else endOfRun = this->runBreaksZ.at( i+1 );

    }

    return true;

}

// copy-operator for backup procedure
bool CAdvancedHRLE::copy_file(std::string input, std::string output)
{
    std::ifstream in(input.c_str());
    if (!in.is_open())
    return false;
    std::ofstream out(output.c_str());
    if (!out.is_open())
    return false;
    return out << in.rdbuf();
}

// rewrites one single part file
bool CAdvancedHRLE::rewritePart( CHierarchicalRLE* part, short int definedIndex ) {

    int     chunkSize, i;

    // setup filename
    string filename = this->workingDirectory;
    filename += "part_";
    int partIndex(definedIndex);
    std::stringstream ss;
    if (ss << partIndex)
    {
        filename += ss.str();
    }
    else {
        cerr << "Error defining filename for partfile!" << endl;
        return false;
    }
    filename += ".dat";

//     cout << "Rewriting file " << filename << endl;

    // open file stream for part file
    std::ofstream partFile( filename.c_str(), ios_base::binary & ios_base::out );

    // add runs of the y coordinate
    partFile.write( "RunY", 4);
    chunkSize = (int) part->run_types.size();
    partFile.write( (char*)&(chunkSize), 4);
    for ( i = 0; i < part->run_types.size(); i++ ) {
        partFile.write( (char*)&(part->run_types.at(i)), 4);
    }

    // add breaks of the y coordinate
    partFile.write( "BrkY", 4);
    chunkSize = (int) part->run_breaks.size();
    partFile.write( (char*)&(chunkSize), 4);
    for ( i = 0; i < part->run_breaks.size(); i++ ) {
        partFile.write( (char*)&(part->run_breaks.at(i)), 2);
    }

    // add runs of the x coordinate
    partFile.write( "RunX", 4);
    chunkSize = (int) part->lower_hrle->run_types.size();
    partFile.write( (char*)&(chunkSize), 4);
    for ( i = 0; i < part->lower_hrle->run_types.size(); i++ ) {
        partFile.write( (char*)&(part->lower_hrle->run_types.at(i)), 4);
    }

    // add breaks of the x coordinate
    partFile.write( "BrkX", 4);
    chunkSize = (int) part->lower_hrle->run_breaks.size();
    partFile.write( (char*)&(chunkSize), 4);
    for ( i = 0; i < part->lower_hrle->run_breaks.size(); i++ ) {
        partFile.write( (char*)&(part->lower_hrle->run_breaks.at(i)), 2);
    }

    // add start coordinates of the x coordinate
    partFile.write( "iStX", 4);
    chunkSize = (int) part->lower_hrle->start_indices.size();
    partFile.write( (char*)&(chunkSize), 4);
    for ( i = 0; i < part->lower_hrle->start_indices.size(); i++ ) {
        partFile.write( (char*)&(part->lower_hrle->start_indices.at(i)), 4);
    }

    partFile.close();

}

// rewrites a sequence of loaded part files
bool CAdvancedHRLE::rewriteLoadedParts( short int startCoord, short int endCoord, CHierarchicalRLE* zEncoding,
                                        int *segmentIndex, short int firstLoadedCoord, short int lastLoadedCoord ) {

    short int           newStartCoord, newEndCoord;
    int                 chunkSize;
    int                 zRun;
    short int           zCoord;
    short int           startOfRun, endOfRun;
    int                 defYIndex = *segmentIndex;
    CHierarchicalRLE*   tmpPart;


    // start and end coordinates of the runs which should be rewritten
    if ( firstLoadedCoord <= this->minZ ) newStartCoord = this->partialHrleObject->min;
    else newStartCoord = startCoord;

    if ( this->partialHrleObject->run_breaks.size() == 0 ) {
        if ( lastLoadedCoord >= this->maxZ ) newEndCoord = this->partialHrleObject->max;
        else /*if ( endCoord < this->maxZ)*/ newEndCoord = endCoord;
    }
    else {
        int i = 0;
        newEndCoord = this->partialHrleObject->min;
        while ( newEndCoord <= newStartCoord ) {
            if (i < this->partialHrleObject->run_breaks.size()) {
                if (lastLoadedCoord < this->maxZ ) {
                    if ( this->partialHrleObject->run_breaks.at(i) < endCoord ) {
                        newEndCoord = this->partialHrleObject->run_breaks.at(i++);
                    }
                    else newEndCoord = endCoord;
                }
                else newEndCoord = this->partialHrleObject->run_breaks.at(i++);
            }
            else {
                if (lastLoadedCoord >= this->maxZ ) newEndCoord = this->partialHrleObject->max;
                else newEndCoord = endCoord;
            }
        }
    }

    // start and end coordinates of the first run in the partial hrle object
    startOfRun = this->partialHrleObject->min;
    if ( this->partialHrleObject->run_breaks.size() == 0 ) endOfRun = this->partialHrleObject->max;
    else endOfRun = this->partialHrleObject->run_breaks.at(0);

    // rewrite the part files
    for ( zRun = 0; zRun < this->partialHrleObject->run_types.size(); zRun++ ) {

        if ( (endOfRun > newStartCoord) && (startOfRun < newEndCoord) ) {

            int run = this->partialHrleObject->run_types.at(zRun);

            if ( RT_DEFINED( run ) ) {

                if( zEncoding->run_types.size() == 0) {
                    zEncoding->run_types.push_back( defYIndex );
                    zEncoding->run_breaks.push_back( newEndCoord );
                }
                else {
                    if ( !RT_DEFINED(zEncoding->run_types.back() ) ) {
                        zEncoding->run_types.push_back( defYIndex );
                    }
                    else {
                        if ( zEncoding->run_breaks.size() != 0 ) zEncoding->run_breaks.pop_back();
                    }
                    zEncoding->run_breaks.push_back( newEndCoord );
                }

                // rewrite the parts
                for ( zCoord = newStartCoord; zCoord < newEndCoord; zCoord++ ) {
                    // calculate defined index of the segment
                    int definedIndex = run + zCoord - startOfRun;
                    tmpPart = this->partialHrleObject->lower_hrle->getSegmentAt( definedIndex );
                    this->rewritePart( tmpPart, defYIndex );
                    delete tmpPart;
                    defYIndex++;
                }

            }
            else {
                if( zEncoding->run_types.size() == 0) {
                    zEncoding->run_types.push_back( run );
                }
                else {
                    if ( RT_DEFINED(zEncoding->run_types.back()) ||
                        (zEncoding->run_types.back() != run) ) {
                            zEncoding->run_types.push_back( run );
                    }
                    else {
                        if ( zEncoding->run_breaks.size() != 0 ) zEncoding->run_breaks.pop_back();
                    }
                }
                zEncoding->run_breaks.push_back( newEndCoord );
            }

            // update start and end coordinates for rewriting
            newStartCoord = newEndCoord;
            if ( (zRun+1) >= this->partialHrleObject->run_breaks.size() ) {
                if (lastLoadedCoord >= this->maxZ ) newEndCoord = this->partialHrleObject->max;
                else newEndCoord = endCoord;
            }
            else {
                if (lastLoadedCoord >= this->maxZ ) {
                    newEndCoord = this->partialHrleObject->run_breaks.at(zRun+1);
                }
                else {
                    if ( this->partialHrleObject->run_breaks.at(zRun+1) < endCoord ) {
                        newEndCoord = this->partialHrleObject->run_breaks.at(zRun+1);
                    }
                    else newEndCoord = endCoord;//this->maxZ;
                }
            }
        }


        // start coordinate of the dilated run in the partial HRLE object
        // to calculate defined index of the y segment
        startOfRun = endOfRun;
        if ( (zRun+1) >= this->partialHrleObject->run_breaks.size() )
            endOfRun = this->partialHrleObject->max;
        else endOfRun = this->partialHrleObject->run_breaks.at(zRun+1);

    }

    // update max/min
    if ( this->partialHrleObject->min < zEncoding->min ) 
        zEncoding->min = this->partialHrleObject->min;
    if ( this->partialHrleObject->max > zEncoding->max ) 
        zEncoding->max = this->partialHrleObject->max;

    if ( this->partialHrleObject->lower_hrle->min < zEncoding->lower_hrle->min ) 
        zEncoding->lower_hrle->min = this->partialHrleObject->lower_hrle->min;
    if ( this->partialHrleObject->lower_hrle->max > zEncoding->lower_hrle->max ) 
        zEncoding->lower_hrle->max = this->partialHrleObject->lower_hrle->max;

    if ( this->partialHrleObject->lower_hrle->lower_hrle->min < zEncoding->lower_hrle->lower_hrle->min ) 
        zEncoding->lower_hrle->lower_hrle->min = this->partialHrleObject->lower_hrle->lower_hrle->min;
    if ( this->partialHrleObject->lower_hrle->lower_hrle->max > zEncoding->lower_hrle->lower_hrle->max ) 
        zEncoding->lower_hrle->lower_hrle->max = this->partialHrleObject->lower_hrle->lower_hrle->max;

    delete this->partialHrleObject;
    this->partialHrleObject = NULL;

    *segmentIndex = defYIndex;

    return true;
}

// rewrite the master file
bool CAdvancedHRLE::rewriteMasterFile( CHierarchicalRLE* zEncoding ) {

    short int           chunkSize;
    short int           i;

    // delete last false break in the dilation of the z coordinate
    zEncoding->run_breaks.pop_back();

    // setup filename
    string masterFileName = this->workingDirectory;
    masterFileName += "master.dat";

    // open file stream for master file
    std::ofstream masterFile( masterFileName.c_str(), ios_base::binary & ios_base::out );

    // maximum and minmum extends for the three dimensions
    masterFile.write( "MinZ", 4);
    masterFile.write( (char*)&(zEncoding->min), 2);
    masterFile.write( "MaxZ", 4);
    masterFile.write( (char*)&(zEncoding->max), 2);
    masterFile.write( "MinY", 4);
    masterFile.write( (char*)&(zEncoding->lower_hrle->min), 2);
    masterFile.write( "MaxY", 4);
    masterFile.write( (char*)&(zEncoding->lower_hrle->max), 2);
    masterFile.write( "MinX", 4);
    masterFile.write( (char*)&(zEncoding->lower_hrle->lower_hrle->min), 2);
    masterFile.write( "MaxX", 4);
    masterFile.write( (char*)&(zEncoding->lower_hrle->lower_hrle->max), 2);

    // add runs of the z coordinate
    masterFile.write( "Runs", 4);
    chunkSize = (short int) zEncoding->run_types.size();
    masterFile.write( (char*)&(chunkSize), 2);
    for ( i = 0; i < zEncoding->run_types.size(); i++ ) {
        masterFile.write( (char*)&(zEncoding->run_types.at(i)), 2);
    }

    // add breaks of the z coordinate
    masterFile.write( "Brks", 4);
    chunkSize = (short int) zEncoding->run_breaks.size();
    masterFile.write( (char*)&(chunkSize), 2);
    for ( i = 0; i < zEncoding->run_breaks.size(); i++ ) {
        masterFile.write( (char*)&(zEncoding->run_breaks.at(i)), 2);
    }

    masterFile.close();

    delete zEncoding;
    zEncoding = NULL;

    this->reloadObjectFile();

    return true;
}

//**************************************************************************************************//
//                                      testing
//**************************************************************************************************//


// draw the whole object
void CAdvancedHRLE::drawTheObject() {

    int               i,j;
    int               definedIndex = 0;
    short int         startOfRun = this->minZ, endOfRun;
    int               run, planeRun;

    float CubeW = (this->maxRange.getX() - this->minRange.getX()) / (float)(this->maxX - this->minX);
    float CubeH = (this->maxRange.getY() - this->minRange.getY()) / (float)(this->maxY - this->minY);
    float CubeD = (this->maxRange.getZ() - this->minRange.getZ()) / (float)(this->maxZ - this->minZ);

    this->loadCompleteData();

    vector<definedPoint> border = this->partialHrleObject->getBorderPoints();

    for ( i = 0; i < border.size(); i++ ) {

                            // draw cubes and cube edges
                            // -------------------------

            CVertex3D point1( this->minRange.getX() + border.at(i).coordinate.at(0) * CubeW,
                              this->minRange.getY() + border.at(i).coordinate.at(1) * CubeH,
                              this->minRange.getZ() + border.at(i).coordinate.at(2) * CubeD );
            CVertex3D point2( this->minRange.getX() + (border.at(i).coordinate.at(0)+1) * CubeW,
                              this->minRange.getY() + (border.at(i).coordinate.at(1)+1) * CubeH,
                              this->minRange.getZ() + (border.at(i).coordinate.at(2)+1) * CubeD);

            this->partialHrleObject->drawACube(&point1, &point2, 1.0);

      }

      delete this->partialHrleObject;
      this->partialHrleObject = NULL;

}

// prints contents of the data structures to standard out
void CAdvancedHRLE::printDataStructure() {

    cout << "Ausgabe der Datenstruktur: " << endl;
    int i;
    cout << "Min X: " << this->minX << endl;
    cout << "Max X: " << this->maxX << endl;
    cout << "Min Y: " << this->minY << endl;
    cout << "Max Y: " << this->maxY << endl;
    cout << "Min Z: " << this->minZ << endl;
    cout << "Max Z: " << this->maxZ << endl;

    cout << endl;
    cout << "Run Types (" << this->runTypesZ.size() << "): ";
    for (i = 0; i < this->runTypesZ.size(); i++) {
        if ( RT_DEFINED(this->runTypesZ.at(i)) ) cout << "d "<< this->runTypesZ.at(i) <<" | ";
        if ( this->runTypesZ.at(i) == RT_NEGATIVE ) cout << "- | ";
        if ( this->runTypesZ.at(i) == RT_POSITIVE ) cout << "+ | ";
    }
    cout << endl;
    cout << "Run Breaks (" << this->runBreaksZ.size() << "): ";
    for (i = 0; i < this->runBreaksZ.size(); i++) cout << this->runBreaksZ.at(i) << " | ";
    cout << endl;


}

// loads the whole object into memory
void CAdvancedHRLE::loadCompleteData() {
    this->loadPartialObject( this->minZ, this->maxZ );
}