/***************************************************************************
 *   Copyright (C) 2007 by Holger Gerth                                    *
 *   holger.gerth@informatik.tu-chemnitz.de                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ADVANCEDHRLE_H
#define ADVANCEDHRLE_H

#include <fstream>
#include <sstream>
#include <vector>

#include "includes.h"
#include "hierarchicalrle.h"
#include "vector3d.h"
#include "vertex3d.h"

/**
	@author Holger Gerth <holger.gerth@informatik.tu-chemnitz.de>
*/
class CAdvancedHRLE{
public:
    CAdvancedHRLE();
    ~CAdvancedHRLE();

    bool reloadObjectFile();
    bool loadObjectFile( string directory );
    bool morphOperation( CHierarchicalRLE* structureElement, vector<short int> qv, short int numZCoords, morphType type );
    bool medialAxis();

    // testing
    void drawTheObject();
    void printDataStructure();
    void loadCompleteData();

private:
    string                              workingDirectory;               // directory which includes the data files
    int                                 minX,maxX,minY,maxY,minZ,maxZ;  // extends of the object
    vector<short int>                   runTypesZ;                      // runs along the z-axis
    vector<short int>                   runBreaksZ;                     // breaks between the z-runs
//     vector<short int>                   startIndicesY;                  // start indices of the y-runs

    CHierarchicalRLE*                   partialHrleObject;              // current loeaded working part of the object

    CHierarchicalRLE* getPlane( int definedIndex, bool backup = false );                  // get the xy-plane of the given defined index
    bool loadPartialObject( short int startCoord, short int endCoord, bool thinning = false, bool backup = false );
    bool backupFiles();
//     short int littleEndian2BigEndian( short int original );
    bool copy_file(std::string input, std::string output);
    bool rewritePart( CHierarchicalRLE* part, short int definedIndex );
    bool rewriteLoadedParts( short int startCoord, short int endCoord, CHierarchicalRLE* zEncoding, int *segmentIndex, short int firstLoadedCoord, short int lastLoadedCoord );
    bool rewriteMasterFile( CHierarchicalRLE* zEncoding );

    // for drawing
    CVector3D                           minRange, maxRange;

};

#endif
