/***************************************************************************
 *   Copyright (C) 2004 by Gerd Gromann                                   *
 *   grog@hrz.tu-chemnitz.de                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ARRAY3D_H
#define ARRAY3D_H

/**
	@author Gerd Gromann <grog@hrz.tu-chemnitz.de>
*/

#include <vector>
using namespace std;


// template class CArray3D
// -----------------------

template <class T> class CArray3D {

    // protected member variables
    // --------------------------

    protected:

        // array scope
        int w, h, d;

        // fake array box
        T fake;

        // data
        vector<T> data;



    // public member functions
    // -----------------------

    public:

//         typename vector<T>::iterator pos;
//         typename vector< vector< T ::iterator pos;

        // construction, destruction
        CArray3D() { 
            data.clear()/*=NULL*/; 
            w=h=d=0; 
//             this->pos = this->data.begin();
        }
        CArray3D(int w, int h, int d) { 
            init(w,h,d); 
//             this->pos = this->data.begin();
        }
        ~CArray3D() { free(); }

        // initialization
        //inline void init(int w, int h, int d);
        inline void init(int w, int h, int d, T initdata);

        // free array
        inline void free();

        inline bool isEmpty();

        // access
        inline T& ref(int i, int j, int k);
        inline T& operator()(int i,int j, int k) { return ref(i,j, k); }
//         inline void initSequentiallAccess();
//         inline bool nextElement();

        // get array scope
        inline int width() const { return w; }
        inline int height() const { return h; }
        inline int depth() const { return d; }

};

//////////////////////////////////////////////////////////////////////
// Initialization
//////////////////////////////////////////////////////////////////////

/*
template<class T> inline void CArray3D<T>::init(int width, int height, int depth) {

    // assign scope
    w = width;
    h = height;
    d = depth;

    // create data array
    data.insert(data.end(), w*h*d,  = new T[w*h*d];

}
*/

template<class T> inline void CArray3D<T>::init(int width, int height, int depth, T initdata) {

    free();

    // assign scope
    w = width;
    h = height;
    d = depth;

    // create data array
    data.insert(data.end(), w*h*d, initdata);

    // set fake value
    fake = initdata;

    /*
    for (int i = 0; i < w; i++) {
        for (int j = 0; j < h; j++) {
            for (int k = 0; k < d; k++) {
                data[k*w*h + j*w + i] = initdata;
            }
        }
    }
    */

}


template<class T> inline bool CArray3D<T>::isEmpty() {

    return (this->data.empty());

}

//////////////////////////////////////////////////////////////////////
// Free array
//////////////////////////////////////////////////////////////////////

template<class T> inline void CArray3D<T>::free() {

    data.clear();
    w = h = d = 0;

    /*
    if (data) {
            delete[] data;
            data = NULL;
            w = h = d = 0;
    }
    */

}

//////////////////////////////////////////////////////////////////////
// Access
//////////////////////////////////////////////////////////////////////

template<class T> inline T& CArray3D<T>::ref(int i, int j, int k) {

    if (!data.empty() && i >= 0 && i < w && j >= 0 && j < h && k >= 0 && k < d)
        return data[k*w*h + j*w + i];
    else
        return fake;

}

//////////////////////////////////////////////////////////////////////
// Sequentiall traversing
//////////////////////////////////////////////////////////////////////

// template<class T> inline void CArray3D<T>::initSequentiallAccess() {
//     this->pos = this->data.begin();
// }
// 
// template<class T> inline bool CArray3D<T>::nextElement() {
//     this->pos++;
// 
//     if ( this->pos == this->data.end() ) return false;
// 
//     return true;
// }

#endif
