// Polygon.cpp: Implementierung der Klasse CPolygon.
//
//////////////////////////////////////////////////////////////////////

#include "polygon.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CPolygon::CPolygon()
{

    vNormalVector.setX( 0.0 );
    vNormalVector.setY( 0.0 );
    vNormalVector.setZ( 0.0 );

    pVertexArray = NULL;
}

CPolygon::CPolygon(vector<CVertex3D>* vertices)
{

    vNormalVector.setX( 0.0 );
    vNormalVector.setY( 0.0 );
    vNormalVector.setZ( 0.0 );

    pVertexArray = vertices;
}

CPolygon::~CPolygon()
{

}


/*!
    \fn CPolygon::GetNormalVector()
 */
// return normal vector of this polygon
CVector3D CPolygon::getNormalVector()
{

    if (pVertexArray != NULL) {

        if (this->iNumVertices >= 3) {
            if ((vNormalVector.length() < 0.99) || (vNormalVector.length() > 1.01))
                this->calcNormalVector();

            return vNormalVector;
        }
        else return CVector3D(0.0,0.0,0.0);

    }
    else return CVector3D(0.0,0.0,0.0);

}

/*!
    \fn CPolygon::CalcNormalVector()
 */
// calculate normal vector of this polygon
void CPolygon::calcNormalVector() {

	// get the vertex indices in the vertex array
	int pos0 = this->iVertexIndex[0];
	int pos1 = this->iVertexIndex[1];
	int pos2 = this->iVertexIndex[2];

	CVertex3D p0 = (*pVertexArray)[pos0];
	CVertex3D p1 = (*pVertexArray)[pos1];
	CVertex3D p2 = (*pVertexArray)[pos2];

	// calculate two vectors which span the plane
	CVector3D a(p1, p0);
	CVector3D b(p1, p2);

	// calculate the cross product
	vNormalVector.setX( a.getY() * b.getZ() - a.getZ() * b.getY() );
	vNormalVector.setY( a.getZ() * b.getX() - a.getX() * b.getZ() );
	vNormalVector.setZ( a.getX() * b.getY() - a.getY() * b.getX() );

	vNormalVector.normalize();

}

unsigned int CPolygon::getNumVertices()
{ 
    return iNumVertices; 
}

void CPolygon::setNumVertices( unsigned int theValue )
{
    this->iNumVertices = theValue;
}

int CPolygon::getVertexIndexAtPos( int pos )
{
    return iVertexIndex.at( pos );
}


void CPolygon::addVertexIndex( int theValue )
{
  iVertexIndex.push_back( theValue );
}


void CPolygon::setVertexArrayPointer(vector< CVertex3D >* theValue)
{
    pVertexArray = theValue;
}
