/***************************************************************************
 *   Qt 4 OpenGL Template                                                  *
 *   Copyright (C) 2007 by Holger Gerth                                    *
 *   holger.gerth@informatik.tu-chemnitz.de                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "qgltemplate.h"

/*!
    \fn qgltemplate::qgltemplate()
 */
qgltemplate::qgltemplate()
    : QMainWindow( 0, 0 )
{

    // set openglWidget size
    this->setGeometry(50,50,800,600);

    // create OpenGL openglWidget
    openglWidget = new CqOpenGLwidget(this);

    // setup geometry for the OpenGL openglWidget
    openglWidget->setGeometry( this->geometry());

    // create the menu
    createMenu();



    setCentralWidget( openglWidget );
    openglWidget->show();
    statusBar()->showMessage( tr("Ready"), 2000 );


}

/*!
    \fn qgltemplate::~qgltemplate()
 */
qgltemplate::~qgltemplate()
{

    delete newAct;
    delete openAct;
    delete saveAct;
    delete saveAsAct;
    delete closeAct;
    delete quitAct;
    delete aboutAct;
    delete aboutQtAct;
//     delete whatsThisAct;

    delete lightingAct;
    delete coordinatesAct;
    delete wireframeAct;
    delete colorAct;
    delete cullingAct;
    delete transparencyAct;
    delete camera0Act;
    delete camera1Act;
    delete triangleDrawAct;

    delete voxelLoadAct;
    delete voxelDrawAct;
    delete voxelMemUsage;
    delete saveHRLEbinaryAction;

    delete hrleLoadAct;
    delete hrleDrawRandAct;
    delete hrleMemUsage;
    delete hrleDilation;
    delete hrleDilation2;
    delete hrleErosion;
    delete hrleMedialSurface;
    delete hrleMedialAxis;

    delete advancedHrleDrawAct;
    delete advancedHrleDilation;
    delete advancedHrleMedialAxis;
    delete advancedHrleErosion;
    delete advancedHrleOpening;
    delete advancedHrleClosing;

    delete loadTestObjectAct;

}

// ******************************************************************************
// slots & functions
// ******************************************************************************

/*!
    \fn qgltemplate::newDoc()
 */
void qgltemplate::newDoc()
{
    qgltemplate *ed = new qgltemplate;
    ed->setWindowTitle(tr("Qt OpenGL Template"));
    ed->show();
}

/*!
    \fn qgltemplate::choose()
 */
// file selector for load
void qgltemplate::choose()
{
    QString fn = QFileDialog::getOpenFileName( this, QString::null, QString::null);
    if ( !fn.isEmpty() )
	load( fn );
    else
	statusBar()->showMessage( tr("Loading aborted"), 2000 );
}

// file selector for load
void qgltemplate::chooseHRLEB()
{
//     QString fn = QFileDialog::getOpenFileName( this, QString::null, QString::null);
    QString fileDirectory = QFileDialog::getExistingDirectory(this, 
            tr("Open Directory"),
            QString::null,
            QFileDialog::ShowDirsOnly |
            QFileDialog::DontResolveSymlinks);
    if ( !fileDirectory.isEmpty() ) {
        if ( openglWidget->loadHRLEB( fileDirectory ) ) {
            this->openglWidget->setHrleBObjectStatus( true );
            this->advancedHrleDrawAct->setEnabled( true );
            this->advancedHrleDilation->setEnabled( true );
            this->advancedHrleErosion->setEnabled( true );
            this->advancedHrleMedialAxis->setEnabled( true );
            this->advancedHrleOpening->setEnabled( true );
            this->advancedHrleClosing->setEnabled( true );
        }
        else
            statusBar()->showMessage( tr("Loading failed"), 2000 );
    }
    else
        statusBar()->showMessage( tr("Loading failed"), 2000 );
}

/*!
    \fn qgltemplate::load( const QString &filename )

    @param filename
 */
// for loading an smf or lwo file
void qgltemplate::load( const QString &fileName )
{
    QStringList parts = fileName.split('.');
    QByteArray tmp = fileName.toAscii();

    if (parts.contains("smf")) {
        openglWidget->loadSmfObject( tmp.constData() );

        this->triangleDrawAct->setEnabled( true );
        this->voxelLoadAct->setEnabled( true );

        statusBar()->showMessage( tr("Loaded source file %1").arg(fileName), 2000 );
    }
    else {
        statusBar()->showMessage( tr("False file format"), 2000 );
        cerr << "Error: False file format!" << endl;
    }
}

/*!
    \fn qgltemplate::save()
 */
void qgltemplate::save()
{
/// @TODO implement save() slot
//     if ( filename.isEmpty() ) {
// 	saveAs();
// 	return;
//     }
// 
//    QString text = e->text();
//     QFile f( filename );
//     if ( !f.open( IO_WriteOnly ) ) {
// 	statusBar()->showMessage( tr("Could not write to %1").arg(filename),
// 			      2000 );
// 	return;
//     }
// 
//     QTextStream t( &f );
//    t << text;
//     f.close();
// 
//    e->setModified( FALSE );
// 
//     setCaption( filename );
// 
//     statusBar()->showMessage( tr( "File %1 saved" ).arg( filename ), 2000 );
}

/*!
    \fn qgltemplate::saveAs()
 */
void qgltemplate::saveAs()
{
/// @TODO implement saveAs() slot
//     QString fn = QFileDialog::getSaveFileName( QString::null, QString::null,
// 					       this );
//     if ( !fn.isEmpty() ) {
// 	filename = fn;
// 	save();
//     } else {
// 	statusBar()->showMessage( tr("Saving aborted"), 2000 );
//     }
}

/*!
    \fn qgltemplate::closeEvent( QCloseEvent* ce )

    @param ce
 */
void qgltemplate::closeEvent( QCloseEvent* ce )
{
/// @TODO implement closeEvent() function (if needed)
//     if ( !e->isModified() ) {
// 	ce->accept();
// 	return;
//     }
// 
//     switch( QMessageBox::information( this, tr("Qt Application Example"),
// 				      tr("Do you want to save the changes"
// 				      " to the document?"),
// 				      tr("Yes"), tr("No"), tr("Cancel"),
// 				      0, 1 ) ) {
//     case 0:
// 	save();
// 	ce->accept();
// 	break;
//     case 1:
// 	ce->accept();
// 	break;
//     case 2:
//     default: // just for sanity
// 	ce->ignore();
// 	break;
//     }
}

/*!
    \fn qgltemplate::about()
 */
void qgltemplate::about()
{
    QMessageBox::about( this, tr("Qt OpenGL Template"),
			tr("Template for OpenGL with Qt 4."
			"\n(C)2006 Holger Gerth"
			"\nholger.gerth@informatik.tu-chemnitz.de"
			"\n\nVersion for testing the H-RLE level set class."));
}

/*!
    \fn qgltemplate::aboutQt()
 */
void qgltemplate::aboutQt()
{
    QMessageBox::aboutQt( this, tr("Qt OpenGL Template") );
}


/*!
    \fn qgltemplate::createMenu()
 */
void qgltemplate::createMenu()
{
    // create the file menu
    QMenu * file = menuBar()->addMenu(tr("&File"));

        // setup the actions
        newAct = new QAction( tr("&New"), this);
        newAct->setShortcut(tr("Ctrl+N"));
        newAct->setStatusTip(tr("Create a new instance"));
        connect(newAct, SIGNAL(triggered()), this, SLOT(newDoc()));

        openAct = new QAction( tr("&Open..."), this);
        openAct->setShortcut(tr("Ctrl+O"));
        openAct->setStatusTip(tr("Open an existing file"));
        connect(openAct, SIGNAL(triggered()), this, SLOT(choose()));

        openHRLEBinAct = new QAction( tr("&Open HRLEB file"), this);
        openHRLEBinAct->setStatusTip(tr("Open a HRLE binary file"));
        connect(openHRLEBinAct, SIGNAL(triggered()), this, SLOT(chooseHRLEB()));

        saveAct = new QAction( tr("&Save"), this);
        saveAct->setShortcut(tr("Ctrl+S"));
        saveAct->setStatusTip(tr("Save the object"));
        connect(saveAct, SIGNAL(triggered()), this, SLOT(save()));
        saveAct->setDisabled(true);

        saveAsAct = new QAction( tr("Save &As"), this);
        saveAsAct->setStatusTip(tr("Save the object as ..."));
        connect(saveAsAct, SIGNAL(triggered()), this, SLOT(saveAs()));
        saveAsAct->setDisabled(true);

        closeAct = new QAction( tr("&Close"), this);
        closeAct->setShortcut(tr("Ctrl+W"));
        closeAct->setStatusTip(tr("Close this Widget"));
        connect(closeAct, SIGNAL(triggered()), this, SLOT(close()));

        quitAct = new QAction( tr("&Quit"), this);
        quitAct->setShortcut(tr("Ctrl+Q"));
        quitAct->setStatusTip(tr("Quit the application"));
        connect(quitAct, SIGNAL(triggered()), qApp, SLOT(quit()));
        quitAct->setWhatsThis(tr("Click this option to create a new file."));

        loadTestObjectAct = new QAction( tr("&Load test object"), this);
        loadTestObjectAct->setStatusTip(tr("Load test object"));
        connect(loadTestObjectAct, SIGNAL(triggered()), this, SLOT(loadTestObject()));
        loadTestObjectAct->setWhatsThis(tr("Load a test HLRE object."));

        // insert the actions into the file menu
        file->addAction( newAct );
        file->addAction( openAct );
        file->addAction( openHRLEBinAct );
        file->addAction( loadTestObjectAct );
//         file->addAction( saveAct );
//         file->addAction( saveAsAct );
        file->addSeparator();
        file->addAction( closeAct );
        file->addAction( quitAct );

        menuBar()->addSeparator();

    // create options menu
    QMenu* options = menuBar()->addMenu( tr( "&Options" ));

        lightingAct = new QAction("&Lighting", this);
        lightingAct->setStatusTip(tr("Activate OpenGL Lighting"));
        connect (lightingAct, SIGNAL (triggered ()), this, SLOT (setLighting ()));
        lightingAct->setCheckable( true );
        lightingAct->setChecked( openglWidget->getLightingStatus() );

        coordinatesAct = new QAction("&Coordinate System", this);
        coordinatesAct->setStatusTip(tr("Activate Coordinate System"));
        connect (coordinatesAct, SIGNAL (triggered ()), this, SLOT (drawCoordinates ()));
        coordinatesAct->setCheckable( true );
        coordinatesAct->setChecked( openglWidget->getCoordinatesStatus() );

        wireframeAct = new QAction("&Wireframe", this);
        wireframeAct->setStatusTip(tr("Activate wire frame mode"));
        connect (wireframeAct, SIGNAL (triggered ()), this, SLOT (setWireFrameOn ()));
        wireframeAct->setCheckable( true );
        wireframeAct->setChecked( openglWidget->getWireFrameStatus() );

        colorAct = new QAction("&ColorMaterial", this);
        colorAct->setStatusTip(tr("Use colormaterial instead of material definitions"));
        connect (colorAct, SIGNAL (triggered ()), this, SLOT (setColorMaterial ()));
        colorAct->setCheckable( true );
        colorAct->setChecked( openglWidget->getColorMaterialStatus() );

        cullingAct = new QAction("&BackFace Culling", this);
        cullingAct->setStatusTip(tr("Activate backface culling"));
        connect (cullingAct, SIGNAL (triggered ()), this, SLOT (setBackFaceCulling ()));
        cullingAct->setCheckable( true );
        cullingAct->setChecked( openglWidget->getCullBackFaceStatus() );

        transparencyAct = new QAction("&Transparency", this);
        transparencyAct->setStatusTip(tr("Activate transparency"));
        QObject::connect (transparencyAct, SIGNAL (triggered ()), this, SLOT (setTransparency ()));
        transparencyAct->setCheckable( true );
        transparencyAct->setChecked( openglWidget->getTransparencyStatus() );

        options->addAction( lightingAct );
        options->addAction( colorAct );
        options->addAction( cullingAct );
        options->addAction( transparencyAct );
        options->addSeparator();
        options->addAction( coordinatesAct );
        options->addAction( wireframeAct );

        menuBar()->addSeparator();

    // create camera menu
    QMenu* cameraMenu = menuBar()->addMenu(tr("&Camera"));

        camera0Act = new QAction("&Free Camera", this);
        camera0Act->setStatusTip(tr("Use the free camera"));
        QObject::connect (camera0Act, SIGNAL (triggered ()), this, SLOT (setCamera1 ()));
        camera0Act->setCheckable( true );

        camera1Act = new QAction("&Centered Camera", this);
        camera1Act->setStatusTip(tr("Use the centered camera"));
        QObject::connect (camera1Act, SIGNAL (triggered ()), this, SLOT (setCamera2 ()));
        camera1Act->setCheckable( true );
        camera1Act->setChecked(true);

        cameraMenu->addAction( camera0Act );
        cameraMenu->addAction( camera1Act );

        menuBar()->addSeparator();

    // create menu for the triangulated object
    QMenu * menuTriangleObject = menuBar()->addMenu(tr("&Object"));

        triangleDrawAct = new QAction( tr("&Draw"), this);
        triangleDrawAct->setStatusTip(tr("Draw the triangulated object"));
        connect(triangleDrawAct, SIGNAL(triggered()), this, SLOT(drawTriangulatedObject()));
        triangleDrawAct->setCheckable( true );
        triangleDrawAct->setDisabled( true );

        menuTriangleObject->addAction( triangleDrawAct );

    // create menu for the voxel object
    QMenu * menuVoxelObject = menuBar()->addMenu(tr("&Voxel Object"));

        voxelLoadAct = new QAction( tr("&Create from triangulated object"), this);
        voxelLoadAct->setStatusTip(tr("Create from triangulated object"));
        connect(voxelLoadAct, SIGNAL(triggered()), this, SLOT(loadVoxelObject()));
        voxelLoadAct->setDisabled( true );

        voxelDrawAct = new QAction( tr("&Draw"), this);
        voxelDrawAct->setStatusTip(tr("Draw the voxel object"));
        connect(voxelDrawAct, SIGNAL(triggered()), this, SLOT(drawVoxelObject()));
        voxelDrawAct->setCheckable( true );
        voxelDrawAct->setDisabled( true );

        voxelMemUsage = new QAction( tr("&Memory and access time"), this);
        voxelMemUsage->setStatusTip(tr("Informations about the data structure"));
        connect(voxelMemUsage, SIGNAL(triggered()), this, SLOT(voMemoryUsage()));
        voxelMemUsage->setDisabled( true );

        saveHRLEbinaryAction = new QAction( tr("&Save as HRLE binary file"), this);
        saveHRLEbinaryAction->setStatusTip(tr("Save this voxel object as a HRLE binary file"));
        connect(saveHRLEbinaryAction, SIGNAL(triggered()), this, SLOT(voSaveAsHRLEB()));
        saveHRLEbinaryAction->setDisabled( true );

        menuVoxelObject->addAction( voxelLoadAct );
        menuVoxelObject->addAction( voxelDrawAct );
        menuVoxelObject->addAction( saveHRLEbinaryAction );
//         menuVoxelObject->addAction( voxelMemUsage );

    // create menu for the H-RLE voxel object
    QMenu * menuHrleObject = menuBar()->addMenu(tr("&H-RLE Object"));

        hrleLoadAct = new QAction( tr("&Create from VoxelObject"), this);
        hrleLoadAct->setStatusTip(tr("Create from VoxelObject"));
        connect(hrleLoadAct, SIGNAL(triggered()), this, SLOT(loadHrleObject()));
        hrleLoadAct->setDisabled( true );

        hrleDrawRandAct = new QAction( tr("&Draw (Random Access)"), this);
        hrleDrawRandAct->setStatusTip(tr("Use Random Access for drawing"));
        connect(hrleDrawRandAct, SIGNAL(triggered()), this, SLOT(drawHrleUsingRandomAccess()));
        hrleDrawRandAct->setCheckable( true );
        hrleDrawRandAct->setDisabled( true );

        hrleDilation = new QAction( tr("&Dilation"), this);
        hrleDilation->setStatusTip(tr("Dilate the H-RLE Object"));
        connect(hrleDilation, SIGNAL(triggered()), this, SLOT(dilateHrle()));
        hrleDilation->setDisabled( true );

        hrleDilation2 = new QAction( tr("&New Dilation"), this);
        hrleDilation2->setStatusTip(tr("The new dilation algorithm"));
        connect(hrleDilation2, SIGNAL(triggered()), this, SLOT(dilateHrleAdvanced()));
        hrleDilation2->setDisabled( true );

        hrleErosion = new QAction( tr("&Erosion"), this);
        hrleErosion->setStatusTip(tr("Erode the H-RLE Object"));
        connect(hrleErosion, SIGNAL(triggered()), this, SLOT(erodeHrle()));
        hrleErosion->setDisabled( true );

        hrleMedialSurface = new QAction( tr("&Medial Surface"), this);
        hrleMedialSurface->setStatusTip(tr("Compute the medial surface"));
        connect(hrleMedialSurface, SIGNAL(triggered()), this, SLOT(medialSurfaceHrle()));
        hrleMedialSurface->setDisabled( true );

        hrleMedialAxis = new QAction( tr("&Medial Axis"), this);
        hrleMedialAxis->setStatusTip(tr("Compute the medial axis"));
        connect(hrleMedialAxis, SIGNAL(triggered()), this, SLOT(medialAxisHrle()));
        hrleMedialAxis->setDisabled( true );

        hrleMemUsage = new QAction( tr("&Memory and access time"), this);
        hrleMemUsage->setStatusTip(tr("Informations about the data structure"));
        connect(hrleMemUsage, SIGNAL(triggered()), this, SLOT(hrleMemoryUsage()));
        hrleMemUsage->setDisabled( true );

        menuHrleObject->addAction( hrleLoadAct );
        menuHrleObject->addAction( hrleDrawRandAct );
        menuHrleObject->addAction( hrleDilation );
        menuHrleObject->addAction( hrleDilation2 );
        menuHrleObject->addAction( hrleErosion );
        menuHrleObject->addAction( hrleMedialSurface );
        menuHrleObject->addAction( hrleMedialAxis );

    // create menu for the advanced H-RLE binary object
    QMenu * menuAdvHrleObject = menuBar()->addMenu(tr("&H-RLE Binary Object"));


        advancedHrleDrawAct = new QAction( tr("&Draw (Random Access)"), this);
        advancedHrleDrawAct->setStatusTip(tr("Use Random Access for drawing"));
        connect(advancedHrleDrawAct, SIGNAL(triggered()), this, SLOT(drawHrleB()));
        advancedHrleDrawAct->setCheckable( true );
        advancedHrleDrawAct->setDisabled( true );

        advancedHrleDilation = new QAction( tr("&Dilation"), this);
        advancedHrleDilation->setStatusTip(tr("Dilate the H-RLE Object"));
        connect(advancedHrleDilation, SIGNAL(triggered()), this, SLOT(dilateHrleB()));
        advancedHrleDilation->setDisabled( true );

        advancedHrleErosion = new QAction( tr("&Erosion"), this);
        advancedHrleErosion->setStatusTip(tr("Erode the H-RLE Object"));
        connect(advancedHrleErosion, SIGNAL(triggered()), this, SLOT(erodeHrleB()));
        advancedHrleErosion->setDisabled( true );

        advancedHrleOpening = new QAction( tr("&Opening"), this);
        advancedHrleOpening->setStatusTip(tr("Open the H-RLE Object"));
        connect(advancedHrleOpening, SIGNAL(triggered()), this, SLOT(openingHrleB()));
        advancedHrleOpening->setDisabled( true );

        advancedHrleClosing = new QAction( tr("&Closing"), this);
        advancedHrleClosing->setStatusTip(tr("Close the H-RLE Object"));
        connect(advancedHrleClosing, SIGNAL(triggered()), this, SLOT(closingHrleB()));
        advancedHrleClosing->setDisabled( true );

        advancedHrleMedialAxis = new QAction( tr("&Medial axis"), this);
        advancedHrleMedialAxis->setStatusTip(tr("Get the medial axis of the object"));
        connect(advancedHrleMedialAxis, SIGNAL(triggered()), this, SLOT(medialAxisHrleB()));
        advancedHrleMedialAxis->setDisabled( true );

        menuAdvHrleObject->addAction( advancedHrleDrawAct );
        menuAdvHrleObject->addAction( advancedHrleDilation );
        menuAdvHrleObject->addAction( advancedHrleErosion );
        menuAdvHrleObject->addAction( advancedHrleOpening );
        menuAdvHrleObject->addAction( advancedHrleClosing );
        menuAdvHrleObject->addAction( advancedHrleMedialAxis );


    // create the help menu
    QMenu * help = menuBar()->addMenu(tr("&Help"));

        aboutAct = new QAction( tr("&About"), this);
        aboutAct->setShortcut(tr("F1"));
        aboutAct->setStatusTip(tr("About this OpenGL template"));
        connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

        aboutQtAct = new QAction( tr("About &Qt"), this);
        aboutQtAct->setStatusTip(tr("About the Qt application development framework"));
        connect(aboutQtAct, SIGNAL(triggered()), this, SLOT(aboutQt()));

//         whatsThisAct = new QAction( tr("What's &This"), this);
//         whatsThisAct->setShortcut(tr("Shift+F1"));
//         whatsThisAct->setStatusTip(tr("Click on objects you're interested in"));
//         connect(whatsThisAct, SIGNAL(triggered()), this, SLOT(whatsThis()));

        help->addAction( aboutAct );
        help->addAction( aboutQtAct );
//         help->addSeparator();
//         help->addAction( whatsThisAct );

}

/*!
    \fn qgltemplate::setLighting()
 */
// slot to activate Lighting
void qgltemplate::setLighting() {

    openglWidget->setLightingStatus( !openglWidget->getLightingStatus() );

    openglWidget->updateGL();
}

/*!
    \fn qgltemplate::drawCoordinates()
 */
// slot to set coordinate system on
void qgltemplate::drawCoordinates() {

    openglWidget->setCoordinatesStatus( !openglWidget->getCoordinatesStatus() );

    openglWidget->updateGL();
}


/*!
    \fn qgltemplate::setCamera1()
 */
// slot to setup the camera mode 1
void qgltemplate::setCamera1()
{
    // use free camera
    openglWidget->setCameraMode( 1 );

    openglWidget->updateGL();

    camera0Act->setChecked(true);
    camera1Act->setChecked(false);
}

/*!
    \fn qgltemplate::setCamera2()
 */
// slot to setup the camera mode 2
void qgltemplate::setCamera2()
{
    // use centered camera
    openglWidget->setCameraMode( 0 );

    openglWidget->updateGL();

    camera0Act->setChecked(false);
    camera1Act->setChecked(true);
}

/*!
    \fn qgltemplate::setWireFrameOn()
 */
// slot to activate Wireframe mode
void qgltemplate::setWireFrameOn()
{

    openglWidget->setWireFrameStatus( !openglWidget->getWireFrameStatus() );

    openglWidget->updateGL();
}

/*!
    \fn qgltemplate::setColorMAterial()
 */
// slot to activate ColorMaterial mode
void qgltemplate::setColorMaterial()
{
    openglWidget->setColorMaterialStatus( !openglWidget->getColorMaterialStatus() );

    openglWidget->updateGL();
}


/*!
    \fn qgltemplate::setBackFaceCulling()
 */
// slot to enable/disable backface culling
void qgltemplate::setBackFaceCulling()
{
    openglWidget->setCullBackFaceStatus( !openglWidget->getCullBackFaceStatus() );

    openglWidget->updateGL();
}

/*!
    \fn qgltemplate::setTransparency()
 */
// slot to enable/disable transparency
void qgltemplate::setTransparency()
{
    openglWidget->setTransparencyStatus( !openglWidget->getTransparencyStatus() );

    openglWidget->updateGL();

}

/*!
    \fn qgltemplate::drawTriangulatedObject()
 */
void qgltemplate::drawTriangulatedObject() {
    if ( openglWidget->getDrawSmfObject() ) openglWidget->setDrawSmfObject( false );
    else openglWidget->setDrawSmfObject( true );
}

/*!
    \fn qgltemplate::loadVoxelObject()
 */
// create a voxel object from the smf object
void qgltemplate::loadVoxelObject() {

    int numCubes = QInputDialog::getInteger( this, "Please insert number of voxels: ", "Please insert number of voxels", 1000 );

    if ( openglWidget->getVoxelObjectStatus() == false ) {
        this->voxelDrawAct->setEnabled( true );
        this->saveHRLEbinaryAction->setEnabled( true );
        this->hrleLoadAct->setEnabled( true );
    }

    openglWidget->loadVoxelObject( numCubes );

    statusBar()->showMessage( tr("VoxelObject created from boundary object"), 2000 );

}

/*!
    \fn qgltemplate::drawVoxelObject()
 */
// draw the voxel object
void qgltemplate::drawVoxelObject() {

    if ( openglWidget->getDrawVoxelObject() ) openglWidget->setDrawVoxelObject( false );
    else openglWidget->setDrawVoxelObject( true );

}

void qgltemplate::voSaveAsHRLEB() {

  QString fileDirectory = QFileDialog::getExistingDirectory(this, tr("Open Directory"),
                                                            "/home",
                                                            QFileDialog::ShowDirsOnly |
                                                            QFileDialog::DontResolveSymlinks);
// getOpenFileName( this, QString::null, QString::null);

  if ( !fileDirectory.isEmpty() )
    this->openglWidget->saveVoxelObjectAsHRLEB( fileDirectory );
  else
    statusBar()->showMessage( tr("You must choose a directory!"), 2000 );
}

/*!
    \fn qgltemplate::loadHrleObject()
 */
// load HRLE object from voxel object
void qgltemplate::loadHrleObject() {

    if ( openglWidget->getHrleObjectStatus() == false ) {
        this->hrleDrawRandAct->setEnabled( true );
        this->hrleDilation->setEnabled( true );
        this->hrleDilation2->setEnabled( true );
        this->hrleErosion->setEnabled( true );
        this->hrleMemUsage->setEnabled( true );
        this->hrleMedialSurface->setEnabled( true );
    }

    openglWidget->loadHrleObjectFromVoxelObject();

    statusBar()->showMessage( tr("VoxelObject converted into a H-RLE voxel object"), 2000 );
}

/*!
    \fn qgltemplate::drawHrleUsingRandomAccess()
 */
// draw the object via random access
void qgltemplate::drawHrleUsingRandomAccess() {
    if ( openglWidget->getDrawHrleRand() ) openglWidget->setDrawHrleRand( false );
    else openglWidget->setDrawHrleRand( true );

    openglWidget->setDrawHrleSeq( false );
}

/*!
    \fn qgltemplate::dilateHrle()
 */
// dilate th HRLE object
void qgltemplate::dilateHrle() {
    CSetupStructElement *structureElementForm = new CSetupStructElement();

    structureElementForm->draw();

    vector<bool> structureElement = structureElementForm->value;


    if ( (openglWidget->getHrleObjectStatus() == true) && ( structureElement.size() > 0 ) ) {
        openglWidget->dilateHRLEObject( structureElement );
    }

    statusBar()->showMessage( tr("HRLE object dilated!"), 2000 );

}

/*!
    \fn qgltemplate::dilateHrleAdvanced()
 */
// dilate th HRLE object
void qgltemplate::dilateHrleAdvanced() {
    CSetupStructElement *structureElementForm = new CSetupStructElement();

    structureElementForm->draw();

    vector<bool> structureElement = structureElementForm->value;


    if ( (openglWidget->getHrleObjectStatus() == true) && ( structureElement.size() > 0 ) ) {
        openglWidget->dilateHRLEObject2( structureElement );
    }

    statusBar()->showMessage( tr("HRLE object dilated!"), 2000 );

}

/*!
    \fn qgltemplate::erodeHrle()
 */
// dilate th HRLE object
void qgltemplate::erodeHrle() {
    CSetupStructElement *structureElementForm = new CSetupStructElement();

    structureElementForm->draw();

    vector<bool> structureElement = structureElementForm->value;


    if ( (openglWidget->getHrleObjectStatus() == true) && ( structureElement.size() > 0 ) ) {
        openglWidget->erodeHRLEObject( structureElement );
    }

    statusBar()->showMessage( tr("HRLE object eroded!"), 2000 );

}

/*!
    \fn qgltemplate::medialSurfaceHrle()
 */
// dilate th HRLE object
void qgltemplate::medialSurfaceHrle() {

    if ( openglWidget->getHrleObjectStatus() == true ) {
        openglWidget->medialSurfaceHRLEObject();
        this->hrleMedialAxis->setEnabled( true );
    }

    statusBar()->showMessage( tr("Medial surface created."), 2000 );

}

/*!
    \fn qgltemplate::medialSurfaceHrle()
 */
// dilate th HRLE object
void qgltemplate::medialAxisHrle() {

    if ( openglWidget->getHrleObjectStatus() == true ) {
        openglWidget->medialAxisHRLEObject();
    }

    statusBar()->showMessage( tr("Medial axis created."), 2000 );

}

void qgltemplate::drawHrleB() {
    if ( openglWidget->getDrawHrleB() ) openglWidget->setDrawHrleB( false );
    else openglWidget->setDrawHrleB( true );

    openglWidget->drawHRLEB();
}


void qgltemplate::dilateHrleB() {

    CSetupStructElement *structureElementForm = new CSetupStructElement();

    structureElementForm->draw();

    vector<bool> structureElement = structureElementForm->value;


    if ( (openglWidget->getHrleBObjectStatus() == true) && ( structureElement.size() > 0 ) ) {
        this->openglWidget->dilateHRLEB( structureElement );
    }

    statusBar()->showMessage( tr("HRLE object dilated!"), 2000 );

}

void qgltemplate::erodeHrleB() {

    CSetupStructElement *structureElementForm = new CSetupStructElement();

    structureElementForm->draw();

    vector<bool> structureElement = structureElementForm->value;


    if ( (openglWidget->getHrleBObjectStatus() == true) && ( structureElement.size() > 0 ) ) {
        this->openglWidget->erodeHRLEB( structureElement );
    }

    statusBar()->showMessage( tr("HRLE object eroded!"), 2000 );

}

void qgltemplate::openingHrleB() {

    CSetupStructElement *structureElementForm = new CSetupStructElement();

    structureElementForm->draw();

    vector<bool> structureElement = structureElementForm->value;


    if ( (openglWidget->getHrleBObjectStatus() == true) && ( structureElement.size() > 0 ) ) {
        this->openglWidget->openHRLEB( structureElement );
    }

    statusBar()->showMessage( tr("HRLE object opened!"), 2000 );

}

void qgltemplate::closingHrleB() {

    CSetupStructElement *structureElementForm = new CSetupStructElement();

    structureElementForm->draw();

    vector<bool> structureElement = structureElementForm->value;


    if ( (openglWidget->getHrleBObjectStatus() == true) && ( structureElement.size() > 0 ) ) {
        this->openglWidget->closeHRLEB( structureElement );
    }

    statusBar()->showMessage( tr("HRLE object closed!"), 2000 );

}

void qgltemplate::medialAxisHrleB() {

    if ( openglWidget->getHrleBObjectStatus() == true ) {
        this->openglWidget->medialAxisHRLEB();
    }

    statusBar()->showMessage( tr("Medial axis created!"), 2000 );

}


void qgltemplate::voMemoryUsage()
{

    double mem = this->openglWidget->getMemoryUsageVoxelobject() / 1024.0;
    double accessTimeRandomAccess = this->openglWidget->getAccessTimeRA_voxelobject();
    double accessAverageTimeRandomAccess = this->openglWidget->getAccessAverageTimeRA_voxelobject();
    double accessTimeSequentialAccess = this->openglWidget->getAccessTimeSA_voxelobject();
    CPoint3D resolution = this->openglWidget->getVoxelobjectResolution();
    int numDefVoxels = this->openglWidget->getNumDefinedVoxels();

    QString message;
    message.append("\nThe full voxel grid uses ");
    if ( mem > 1024.0) {
        mem = mem /1024.0;
        message.append(QString::number(mem));
        message.append(" MBytes of system memory.");
    }
    else {
        message.append(QString::number(mem));
        message.append(" KBytes of system memory.");
    }

    message.append("\n\nLast Random Access was done in ");
    message.append(QString::number(accessTimeRandomAccess));
    message.append(" micro seconds.");

    message.append("\nAverage time for Random Access was ");
    message.append(QString::number(accessAverageTimeRandomAccess));
    message.append(" micro seconds.");

    message.append("\n\nLast Sequential Access was done in ");
    if ( accessTimeSequentialAccess > 1000000.0 ) {
        accessTimeSequentialAccess = accessTimeSequentialAccess / 1000000.0;
        message.append(QString::number(accessTimeSequentialAccess));
        message.append(" seconds.");
    }
    else if ( accessTimeSequentialAccess > 1000.0) {
        accessTimeSequentialAccess = accessTimeSequentialAccess / 1000.0;
        message.append(QString::number(accessTimeSequentialAccess));
        message.append(" milli seconds.");
    }
    else {
        message.append(QString::number(accessTimeSequentialAccess));
        message.append(" micro seconds.");
    }

    message.append("\n\nThis voxel object has a resolution of (");
    message.append(QString::number(resolution.getX()));
    message.append(", ");
    message.append(QString::number(resolution.getY()));
    message.append(", ");
    message.append(QString::number(resolution.getZ()));
    message.append(") voxels with ");
    message.append(QString::number(numDefVoxels));
    message.append(" defined voxels.");

    QMessageBox::information( this, tr("Information about memory and access time"), message);
}

void qgltemplate::hrleMemoryUsage()
{

    double mem = this->openglWidget->getMemoryUsageHRLE() / 1024.0;
    double accessTimeRandomAccess = this->openglWidget->getAccessTimeRA();
    double accessAverageTimeRandomAccess = this->openglWidget->getAccessAverageTimeRA();
    double accessTimeSequentialAccess = this->openglWidget->getAccessTimeSA();

    QString message;
    message.append("\nThe HRLE Object uses ");
    if ( mem > 1024.0) {
        mem = mem /1024.0;
        message.append(QString::number(mem));
        message.append(" MBytes of system memory.");
    }
    else {
        message.append(QString::number(mem));
        message.append(" KBytes of system memory.");
    }

    message.append("\n\nLast Random Access was done in ");
    message.append(QString::number(accessTimeRandomAccess));
    message.append(" micro seconds.");

    message.append("\nAverage time for Random Access was ");
    message.append(QString::number(accessAverageTimeRandomAccess));
    message.append(" micro seconds.");

    message.append("\n\nLast Sequential Access was done in ");
    if ( accessTimeSequentialAccess > 1000000.0 ) {
        accessTimeSequentialAccess = accessTimeSequentialAccess / 1000000.0;
        message.append(QString::number(accessTimeSequentialAccess));
        message.append(" seconds.");
    }
    else if ( accessTimeSequentialAccess > 1000.0) {
        accessTimeSequentialAccess = accessTimeSequentialAccess / 1000.0;
        message.append(QString::number(accessTimeSequentialAccess));
        message.append(" milli seconds.");
    }
    else {
        message.append(QString::number(accessTimeSequentialAccess));
        message.append(" micro seconds.");
    }

    QMessageBox::information( this, tr("Information about memory and access time"), message);
}


// *****************************************************************************
// Some events
// *****************************************************************************

void qgltemplate::resizeEvent( QResizeEvent *ev)
{
    QSize newsize = ev->size();

    openglWidget->resize( newsize.width(), newsize.height() );

}

/*!
    \fn qgltemplate::keyPressEvent( QKeyEvent *ev )
 */
void qgltemplate::keyPressEvent( QKeyEvent *ev )
{
    // if key is pressed
    switch ( ev->key() ) {
        default:    break;

    }

    openglWidget->updateGL();
	
}

/*!
    \fn qgltemplate::keyReleaseEvent( QKeyEvent *ev )
 */
void qgltemplate::keyReleaseEvent( QKeyEvent *ev )
{
// if key is released
    switch ( ev->key() ) {
        default:    break;

    }

    openglWidget->updateGL();

}


/////////////////////////////////////////////
//  Testing only
/////////////////////////////////////////////

void qgltemplate::loadTestObject()
{
    openglWidget->createTestingHRLE();
    this->hrleDrawRandAct->setEnabled( true );
    this->hrleLoadAct->setDisabled( true );
    this->hrleDilation->setEnabled( true );
    this->hrleDilation2->setEnabled( true );
    this->hrleErosion->setEnabled( true );
    this->hrleMemUsage->setEnabled( true );
    this->hrleMedialSurface->setEnabled( true );
}



