/***************************************************************************
 *   Copyright (C) 2007 by Holger Gerth                                    *
 *   holger.gerth@informatik.tu-chemnitz.de                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef QGLTEMPLATE_H
#define QGLTEMPLATE_H

#include <qmainwindow.h>
#include "qopenglwidget.h"
//Added by qt3to4:
#include <QCloseEvent>
#include <qimage.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qmenu.h>
#include <qmenubar.h>
#include <qtextedit.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qprinter.h>
#include <qapplication.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qaction.h>
#include <qwhatsthis.h>
#include <QInputDialog>

#include "nbsForm.h"
#include "structElement.h"
// #include "hierarchicalrle.h"

class qgltemplate: public QMainWindow
{
    Q_OBJECT

public:
    qgltemplate();
    ~qgltemplate();

protected:
    void closeEvent( QCloseEvent* );
    void createMenu();
    virtual void resizeEvent( QResizeEvent *ev);

private slots:
    // file menu
    void newDoc();
    void choose();
    void load( const QString &fileName );
    void save();
    void saveAs();
    void chooseHRLEB();

    // help menu
    void about();
    void aboutQt();

    // optiona menu
    void setLighting();
    void drawCoordinates();
    void setCamera1();
    void setCamera2();
    void setWireFrameOn();
    void setColorMaterial();
    void setBackFaceCulling();
    void setTransparency();

    // object menu
    void drawTriangulatedObject();

    // voxel menu
    void loadVoxelObject();
    void drawVoxelObject();
    void voMemoryUsage();
    void voSaveAsHRLEB();

    // hrle menu
    void loadHrleObject();
    void drawHrleUsingRandomAccess();
    void dilateHrle();
    void dilateHrleAdvanced();
    void erodeHrle();
    void hrleMemoryUsage();
    void medialSurfaceHrle();
    void medialAxisHrle();

    // binary hrle menu
    void drawHrleB();
    void dilateHrleB();
    void erodeHrleB();
    void medialAxisHrleB();
    void openingHrleB();
    void closingHrleB();

    // testing
    void loadTestObject();


private:
    QString filename;

    // QActions for the menu
    QAction*         newAct;
    QAction*         openAct;
    QAction*         openHRLEBinAct;
    QAction*         saveAct;
    QAction*         saveAsAct;
    QAction*         closeAct;
    QAction*         quitAct;

    QAction*         loadTestObjectAct;

    // help menu
    QAction*         aboutAct;
    QAction*         aboutQtAct;
//     QAction*         whatsThisAct;

    // options menu
    QAction*        lightingAct;
    QAction*        coordinatesAct;
    QAction*        wireframeAct;
    QAction*        colorAct;
    QAction*        cullingAct;
    QAction*        transparencyAct;

    // camera menu
    QAction*        camera0Act;
    QAction*        camera1Act;

    // menu for the triangulated object
    QAction*        triangleDrawAct;

    // menu for the voxel object
    QAction*        voxelLoadAct;
    QAction*        voxelDrawAct;
    QAction*        voxelMemUsage;
    QAction*        saveHRLEbinaryAction;

    // menu for the hrle object
    QAction*        hrleLoadAct;
    QAction*        hrleDrawRandAct;
    QAction*        hrleDilation;
    QAction*        hrleDilation2;
    QAction*        hrleErosion;
    QAction*        hrleMemUsage;
    QAction*        hrleMedialSurface;
    QAction*        hrleMedialAxis;

    // menu for the hrle object
    QAction*        advancedHrleDrawAct;
    QAction*        advancedHrleDilation;
    QAction*        advancedHrleErosion;
    QAction*        advancedHrleMedialAxis;
    QAction*        advancedHrleOpening;
    QAction*        advancedHrleClosing;


    // OpenGL Widget
    CqOpenGLwidget* openglWidget;

    // virtual functions for key events
    virtual void keyPressEvent( QKeyEvent *ev);
    virtual void keyReleaseEvent( QKeyEvent *ev );

};


#endif
