/***************************************************************************
 *   Copyright (C) 2006 by Holger Gerth                                      *
 *   holger.gerth@informatik.tu-chemnitz.de                                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "smfobject.h"

CsmfObject::CsmfObject()
{

    iFaceCount = 0;
    iVertexCount = 0;
}

CsmfObject::CsmfObject( const char* filename )
{

    iFaceCount = 0;
    iVertexCount = 0;
    loadObjectFile( filename );
}


CsmfObject::~CsmfObject()
{
}

bool CsmfObject::loadObjectFile( const char* filename )
{

    char c;
    char cStr[50];

    // open .smf file
    ifstream file( filename );
    if (!file) { 
        cerr << "Error. Cannot open file " << filename << " !!!" << endl;
        return false;
    }

    // read the file content
    if (!file.good()) {
        cerr << "Error while opening file " << filename << " << !!!" << endl;
        return false;
    }
    else {
        while ( file.good() ) {

            file >> c;

            switch ( c ) {

                case 'v':   double x, y, z;

                            file.ignore(1);
                            file.getline( cStr, 50, ' ' );
                            x = atof( cStr );

                            file.getline( cStr, 50, ' ' );
                            y = atof( cStr );

                            file.getline( cStr, 50, '\n' );
                            z = atof( cStr );

                            addVertex( x, y, z );
                            break;
                case 'f':   int a, b, c;

                            file.ignore(1);
                            file.getline( cStr, 50, ' ' );
                            a = atoi( cStr ) -1;

                            file.getline( cStr, 50, ' ' );
                            b = atoi( cStr ) -1;

                            file.getline( cStr, 50, '\n' );
                            c = atoi( cStr ) -1;

                            addFace( a, b, c );
                            break;
                default:    file.getline( cStr, 50, '\n' );
                            break;

            }

        }
    }

    file.close();

    return true;

}




/*!
    \fn CsmfObject::addVertex( double x, double y, double z );
 */
// push vertex data
void CsmfObject::addVertex( float x, float y, float z )
{
    CVertex3D newVertex( x, y, z );

    vVertices.push_back( newVertex );

    iVertexCount++;
}


/*!
    \fn CsmfObject::addFace( int a, int b, int c )
 */
// push face data
void CsmfObject::addFace( int a, int b, int c )
{
    if ((a >= 0) && (b >= 0) && (c >= 0)) {

        CPolygon newPoly( &(this->vVertices) );;

        newPoly.addVertexIndex( a );
        newPoly.addVertexIndex( b );
        newPoly.addVertexIndex( c );
        newPoly.setNumVertices( 3 );

        vFaces.push_back( newPoly );

        iFaceCount++;
    }
}


unsigned int CsmfObject::getFaceCount() const
{
  return iFaceCount;
}

unsigned int CsmfObject::getVertexCount() const
{
  return iVertexCount;
}

CVertex3D CsmfObject::getVertexAtPos( int pos ) const
{
    if (pos < this->iVertexCount )
        return ( this->vVertices.at( pos ) );
    else cerr << "Out of range ... Vertex at position " << pos << " don't exisit." << endl;
}

CPolygon CsmfObject::getFaceAtPos( int pos ) const
{
    if (pos < this->iFaceCount )
        return ( this->vFaces.at( pos ) );
    else cerr << "Out of range ... Face at position " << pos << " don't exisit." << endl;
}

bool CsmfObject::DoRecalculateBoundingbox()
{
    int size = this->iVertexCount;
    if (size == 0) return false;

    float xmin = vVertices[0].getX();
    float ymin = vVertices[0].getY();
    float zmin = vVertices[0].getZ();

    float xmax = vVertices[0].getX();
    float ymax = vVertices[0].getY();
    float zmax = vVertices[0].getZ();

    for (int i = 1; i < size; i++) {
            if (vVertices[i].getX() < xmin) xmin = vVertices[i].getX();
            if (vVertices[i].getY() < ymin) ymin = vVertices[i].getY();
            if (vVertices[i].getZ() < zmin) zmin = vVertices[i].getZ();

            if (vVertices[i].getX() > xmax) xmax = vVertices[i].getX();
            if (vVertices[i].getY() > ymax) ymax = vVertices[i].getY();
            if (vVertices[i].getZ() > zmax) zmax = vVertices[i].getZ();
    }

    this->vBB_lowerCorner.setX( xmin );
    this->vBB_lowerCorner.setY( ymin );
    this->vBB_lowerCorner.setZ( zmin );

    this->vBB_upperCorner.setX( xmax );
    this->vBB_upperCorner.setY( ymax );
    this->vBB_upperCorner.setZ( zmax );

//     cout << "min: " << xmin << ", " << ymin << ", " << zmin << endl;
//     cout << "max: " << xmax << ", " << ymax << ", " << zmax << endl;

    return true;
}


CVertex3D CsmfObject::getVBB_lowerCorner() const
{
  return vBB_lowerCorner;
}


CVertex3D CsmfObject::getVBB_upperCorner() const
{
  return vBB_upperCorner;
}

void CsmfObject::draw() {

    glBegin(GL_TRIANGLES);

    for (int i = 0; i < this->iFaceCount; i++) {

        CPolygon face = this->vFaces.at(i);

        int pos1 = face.getVertexIndexAtPos( 0 );
        int pos2 = face.getVertexIndexAtPos( 1 );
        int pos3 = face.getVertexIndexAtPos( 2 );

        CVector3D normalVector = this->calcNormal( CVector3D(vVertices[pos1], vVertices[pos2]), 
                                                   CVector3D(vVertices[pos2], vVertices[pos3]) );

        glNormal3f( normalVector.getX(), normalVector.getY(), normalVector.getZ() );
        glVertex3f(vVertices[pos1].getX(), vVertices[pos1].getY(), vVertices[pos1].getZ() );
        glVertex3f(vVertices[pos2].getX(), vVertices[pos2].getY(), vVertices[pos2].getZ() );
        glVertex3f(vVertices[pos3].getX(), vVertices[pos3].getY(), vVertices[pos3].getZ() );

//         cout << "Face: " << i << endl;
//         cout << "Vertex: " << vVertices[pos1].getX() << ", " << vVertices[pos1].getY() << ", " << vVertices[pos1].getZ() << endl;
//         cout << "Vertex: " << vVertices[pos2].getX() << ", " << vVertices[pos2].getY() << ", " << vVertices[pos2].getZ() << endl;
//         cout << "Vertex: " << vVertices[pos3].getX() << ", " << vVertices[pos3].getY() << ", " << vVertices[pos3].getZ() << endl;

    }

    glEnd();

}

// calculate the normal vector for a face
CVector3D CsmfObject::calcNormal( CVector3D a, CVector3D b ) {

    CVector3D returnValue;

    returnValue.setX( a.getY() * b.getZ() - a.getZ() * b.getY() );
    returnValue.setY( a.getZ() * b.getX() - a.getX() * b.getZ() );
    returnValue.setZ( a.getX() * b.getY() - a.getY() * b.getX() );

    returnValue.normalize();

    return returnValue;

}

